
class ServiceEndpoints

  def initialize(protocol = "http", server_root = "localhost:8080", server_path = "oneapiserver", api_version = "1", services = {})
    services = {
      "AmountChargeService"            =>"AmountChargeService",
      "AmountRefundService"            =>"AmountRefundService",
      "AmountReserveService"           =>"AmountReserveService",
      "AmountReserveAdditionalService" =>"AmountReserveAdditionalService",
      "AmountReserveChargeService"     =>"AmountReserveChargeService",
      "AmountReserveReleaseService"    =>"AmountReserveReleaseService",
      "LocationService"                =>"LocationService",
      "SendSMSService"                 =>"SendSMSService",
      "QuerySMSService"                =>"QuerySMSService",
      "SMSDeliveryService"             =>"SMSDeliveryService",
      "CancelSMSDeliveryService"       =>"CancelSMSDeliveryService",
      "RetrieveSMSService"             =>"RetrieveSMSService",
      "SMSReceiptService"              =>"SMSReceiptService",
      "CancelSMSReceiptService"        =>"CancelSMSReceiptService",
      "SendMMSService"                 =>"SendMMSService",
      "QueryMMSService"                =>"QueryMMSService",
      "MMSDeliveryService"             =>"MMSDeliveryService",
      "CancelMMSDeliveryService"       =>"CancelMMSDeliveryService",
      "RetrieveMMSService"             =>"RetrieveMMSService",
      "RetrieveMMSMessageService"      =>"RetrieveMMSMessageService",
      "MMSReceiptService"              =>"MMSReceiptService",
      "CancelMMSReceiptService"        =>"CancelMMSReceiptService"
    }.merge(services)

    base_path  = "#{protocol}://#{server_root}/#{server_path}"

    @AmountCharge                  ="#{base_path}/#{services["AmountChargeService"]}/#{api_version}/payment/{endUserId}/transactions/amount"
    @AmountRefund                  ="#{base_path}/#{services["AmountRefundService"]}/#{api_version}/payment/{endUserId}/transactions/amount"
    @AmountReserve                 ="#{base_path}/#{services["AmountReserveService"]}/#{api_version}/payment/{endUserId}/transactions/amountReservation"
    @AmountReserveAdditional       ="#{base_path}/#{services["AmountReserveAdditionalService"]}/#{api_version}/payment/{endUserId}/transactions/amountReservation/{transactionId}"
    @AmountReservationCharge       ="#{base_path}/#{services["AmountReserveChargeService"]}/#{api_version}/payment/{endUserId}/transactions/amountReservation/{transactionId}"
    @AmountReservationRelease      ="#{base_path}/#{services["AmountReserveReleaseService"]}/#{api_version}/payment/{endUserId}/transactions/amountReservation/{transactionId}"
    @Location                      ="#{base_path}/#{services["LocationService"]}/#{api_version}/location/queries/location"
    @SendSMS                       ="#{base_path}/#{services["SendSMSService"]}/#{api_version}/smsmessaging/outbound/{senderAddress}/requests"
    @QuerySMSDelivery              ="#{base_path}/#{services["QuerySMSService"]}/#{api_version}/smsmessaging/outbound/{senderAddress}/requests/{requestId}/deliveryInfos"
    @SMSDeliverySubscriptions      ="#{base_path}/#{services["SMSDeliveryService"]}/#{api_version}/smsmessaging/outbound/{senderAddress}/subscriptions"
    @CancelSMSDeliverySubscription ="#{base_path}/#{services["CancelSMSDeliveryService"]}/#{api_version}/smsmessaging/outbound/subscriptions/{subscriptionId}"
    @RetrieveSMS                   ="#{base_path}/#{services["RetrieveSMSService"]}/#{api_version}/smsmessaging/inbound/registrations/{registrationId}/messages/?maxBatchSize={maxBatchSize}"
    @SMSReceiptSubscriptions       ="#{base_path}/#{services["SMSReceiptService"]}/#{api_version}/smsmessaging/inbound/subscriptions"
    @CancelSMSReceiptSubscription  ="#{base_path}/#{services["CancelSMSReceiptService"]}/#{api_version}/smsmessaging/inbound/subscriptions/{subscriptionId}"
    @SendMMS                       ="#{base_path}/#{services["SendMMSService"]}/#{api_version}/messaging/outbound/{senderAddress}/requests"
    @QueryMMSDelivery              ="#{base_path}/#{services["QueryMMSService"]}/#{api_version}/messaging/outbound/{senderAddress}/requests/{requestId}/deliveryInfos"
    @MMSDeliverySubscriptions      ="#{base_path}/#{services["MMSDeliveryService"]}/#{api_version}/messaging/outbound/{senderAddress}/subscriptions"
    @CancelMMSDeliverySubscription ="#{base_path}/#{services["CancelMMSDeliveryService"]}/#{api_version}/messaging/outbound/subscriptions/{subscriptionId}"
    @RetrieveMMS                   ="#{base_path}/#{services["RetrieveMMSService"]}/#{api_version}/messaging/inbound/registrations/{registrationId}/messages/?maxBatchSize={maxBatchSize}"
    @RetrieveMMSMessage            ="#{base_path}/#{services["RetrieveMMSMessageService"]}/#{api_version}/messaging/inbound/registrations/{registrationId}/messages/{messageId}?resFormat={resFormat}"
    @MMSReceiptSubscriptions       ="#{base_path}/#{services["MMSReceiptService"]}/#{api_version}/messaging/inbound/subscriptions"
    @CancelMMSReceiptSubscription  ="#{base_path}/#{services["CancelMMSReceiptService"]}/#{api_version}/messaging/inbound/subscriptions/{subscriptionId}"
  end

  def getAmountChargeEndpoint
    @AmountCharge
  end

  def getAmountRefundEndpoint
    @AmountRefund
  end

  def getAmountReserveEndpoint
    @AmountReserve
  end

  def getAmountReserveAdditionalEndpoint
    @AmountReserveAdditional
  end

  def getAmountReservationChargeEndpoint
    @AmountReservationCharge
  end

  def getAmountReservationReleaseEndpoint
    @AmountReservationRelease
  end

  def getLocationEndpoint
    @Location;
  end

  def getSendSMSEndpoint
    @SendSMS
  end

  def getQuerySMSDeliveryEndpoint
    @QuerySMSDelivery
  end

  def getSMSDeliverySubscriptionsEndpoint
    @SMSDeliverySubscriptions
  end

  def getCancelSMSDeliverySubscriptionEndpoint
    @CancelSMSDeliverySubscription
  end

  def getRetrieveSMSEndpoint
    @RetrieveSMS
  end

  def getSMSReceiptSubscriptionsEndpoint
    @SMSReceiptSubscriptions
  end

  def getCancelSMSReceiptSubscriptionEndpoint
    @CancelSMSReceiptSubscription
  end

  def getSendMMSEndpoint
    @SendMMS
  end

  def getQueryMMSDeliveryEndpoint
    @QueryMMSDelivery
  end

  def getMMSDeliverySubscriptionsEndpoint
    @MMSDeliverySubscriptions
  end

  def getCancelMMSDeliverySubscriptionEndpoint
    @CancelMMSDeliverySubscription
  end

  def getRetrieveMMSEndpoint
    @RetrieveMMS
  end

  def getRetrieveMMSMessageEndpoint
    @RetrieveMMSMessage
  end

  def getMMSReceiptSubscriptionsEndpoint
    @MMSReceiptSubscriptions
  end

  def getCancelMMSReceiptSubscriptionEndpoint
    @CancelMMSReceiptSubscription
  end
end

